@@ -38,7 +38,7 @@ import threading
 import re
 import gobject
 from utils import HTMLManipulations
-from lxml.html import fromstring, tostring
+from lxml.html import fromstring, tostring, submit_form
 
 #ugly workaround...
 user_token = [None]
@@ -89,13 +89,13 @@ class CacheDownloader(gobject.GObject):
         return c
                 
     # Update a single coordinate
-    def update_coordinate(self, coordinate, num_logs = 20, outfile = None):
+    def update_coordinate(self, coordinate, num_logs = 20):
         if not CacheDownloader.lock.acquire(False):
             self.emit('already-downloading-error', Exception("There's a download in progress. Please wait."))
             return
         try:
             logger.info("Downloading %s..." % (coordinate.name))
-            u = self._update_coordinate(coordinate, outfile = outfile, num_logs = num_logs)
+            u = self._update_coordinate(coordinate, num_logs = num_logs)
         except Exception, e:
             logger.exception(e)
             CacheDownloader.lock.release()
@@ -142,7 +142,7 @@ class GeocachingComCacheDownloader(CacheDownloader):
     
     MAX_REC_DEPTH = 3
 
-    MAX_DOWNLOAD_NUM = 80
+    MAX_DOWNLOAD_NUM = 800
 
 
     CTIDS = {
@@ -155,20 +155,30 @@ class GeocachingComCacheDownloader(CacheDownloader):
         137:GeocacheCoordinate.TYPE_EARTH
     }
     
+    # URL for log pages; fetches 10 logs by default
+    LOGBOOK_URL = 'http://www.geocaching.com/seek/geocache.logbook?tkn=%s&idx=%d&num=10&decrypt=true'
+    OVERVIEW_URL = 'http://www.geocaching.com/seek/nearest.aspx?lat=%f&lng=%f&dist=%f'
+    PRINT_PREVIEW_URL = 'http://www.geocaching.com/seek/cdpf.aspx?guid=%s'
+    SEEK_URL = "http://www.geocaching.com/seek/%s"
+    DETAILS_URL = 'http://www.geocaching.com/seek/cache_details.aspx?wp=%s'
+    LOGIN_URL = 'https://www.geocaching.com/login/default.aspx'
+    NEAREST_URL = 'http://www.geocaching.com/seek/nearest.aspx'
+    USER_TOKEN_URL = 'http://www.geocaching.com/map/default.aspx?lat=6&lng=9'
+    UPLOAD_FIELDNOTES_URL = 'http://www.geocaching.com/my/uploadfieldnotes.aspx'
+    
     def __init__(self, downloader, path = None, download_images = True):
         logger.info("Using new downloader.")
         CacheDownloader.__init__(self, downloader, path, download_images)
         self.downloader.allow_minified_answers = True
 
     def _parse(self, text):
-        text2 = "%s test" % text
         try:
             t = unicode(text, 'utf-8')
             doc = fromstring(t)
         except Exception, e:
             logger.exception(e)
             from lxml.html.soupparser import fromstring as fromstring_soup
-            doc = fromstring_soup(text2)
+            doc = fromstring_soup(text)
         return doc
 
     def _get_overview(self, location, rec_depth = 0):
@@ -178,20 +188,26 @@ class GeocachingComCacheDownloader(CacheDownloader):
         logger.debug("Distance is %f meters" % dist)
         if dist > 100:
             raise Exception("Please select a smaller part of the map!")
-        url = 'http://www.geocaching.com/seek/nearest.aspx?lat=%f&lng=%f&dist=%f' % (center.lat, center.lon, dist)
+        url = self.OVERVIEW_URL % (center.lat, center.lon, dist)
         response = self.downloader.get_reader(url, login_callback = self.login_callback, check_login_callback = self.check_login_callback)
         
-        points = []
         cont = True
+        wpts = []
+        page_last = 0 # Stores the "old" value of the page counter; If it doesn't increment, abort!
         while cont:
             # Count the number of results and pages
             text = response.read()
+            response.close()
             doc = self._parse(text)
             bs = doc.cssselect('#ctl00_ContentBody_ResultsPanel .PageBuilderWidget b')
             if len(bs) == 0:
                 raise Exception("There are no geocaches in this area.")
             count = int(bs[0].text_content())
             page_current = int(bs[1].text_content())
+            if page_current == page_last:
+                raise Exception("Current page has the same number as the last page; aborting!")
+                break
+            page_last = page_current
             page_max = int(bs[2].text_content())
             logger.info("We are at page %d of %d, total %d geocaches" % (page_current, page_max, count))
             if count > self.MAX_DOWNLOAD_NUM:
@@ -199,7 +215,7 @@ class GeocachingComCacheDownloader(CacheDownloader):
                 
                 
             # Extract waypoint information from the page
-            wpts = [(
+            w = [(
                 # Get the GUID from the link
                 x.getparent().getchildren()[0].get('href').split('guid=')[1], 
                 # See whether this cache was found or not
@@ -207,51 +223,53 @@ class GeocachingComCacheDownloader(CacheDownloader):
                 # Get the GCID from the text
                 x.text_content().split('|')[1].strip()
                 ) for x in doc.cssselect(".SearchResultsTable .Merge .small")]
-                
-            # Download the geocaches using the print preview 
-            for guid, found, id in wpts:
-                coordinate = GeocacheCoordinate(-1, -1, id)
-                coordinate.found = found
-                self.emit("progress", "Geocachepreview", len(points), count)
-                logger.info("Downloading %s..." % id)
-                url = 'http://www.geocaching.com/seek/cdpf.aspx?guid=%s' % guid
-                response = self.downloader.get_reader(url, login_callback = self.login_callback, check_login_callback = self.check_login_callback)                
-                result = self._parse_cache_page_print(response, coordinate, num_logs = 20)
-                if result != None and result.lat != -1:
-                    points += [result]
+            wpts += w
                 
             cont = False  
             # There are more pages...
             if page_current < page_max:
                 from urllib import urlencode
-                doc.forms[0].fields['__EVENTTARGET'] = 'ctl00$ContentBody$pgrTop$lbGoToPage_%d' % (page_current + 1)
-                values = urlencode(doc.forms[0].form_values())
-                action = "http://www.geocaching.com/seek/%s" % doc.forms[0].action
+                doc.forms[0].fields['__EVENTTARGET'] = 'ctl00$ContentBody$pgrTop$ctl08'
+                # Quick hack. Nicer solution would be to remove the element.
+                v = [x for x in doc.forms[0].form_values() if x[0] != 'ctl00$ContentBody$chkAll']
+                values = urlencode(v)
+                action = self.SEEK_URL % doc.forms[0].action
                 logger.info("Retrieving next page!")
                 response = self.downloader.get_reader(action, data=('application/x-www-form-urlencoded', values), login_callback = self.login_callback, check_login_callback = self.check_login_callback)
+                
                 cont = True
-            
+        
+        # Download the geocaches using the print preview 
+        points = []
+        i = 0
+        for guid, found, id in wpts:
+            i += 1
+            coordinate = GeocacheCoordinate(-1, -1, id)
+            coordinate.found = found
+            self.emit("progress", "Geocache", i, len(wpts))
+            logger.info("Downloading %s..." % id)
+            url = self.PRINT_PREVIEW_URL % guid
+            response = self.downloader.get_reader(url, login_callback = self.login_callback, check_login_callback = self.check_login_callback)                
+            result = self._parse_cache_page_print(response, coordinate, num_logs = 20)
+            if result != None and result.lat != -1:
+                points += [result]
                 
         return points
         
-    def _update_coordinate(self, coordinate, num_logs = 20, outfile = None):
+    def _update_coordinate(self, coordinate, num_logs = 20):
         coordinate = coordinate.clone()
         
-        url = 'http://www.geocaching.com/seek/cache_details.aspx?wp=%s' % coordinate.name
+        url = self.DETAILS_URL % coordinate.name
         response = self.downloader.get_reader(url, login_callback = self.login_callback, check_login_callback = self.check_login_callback)
-        if outfile != None:
-            f = open(outfile, 'w')
-            f.write(response.read())
-            f.close()
-            response = open(outfile)
             
         return self._parse_cache_page(response, coordinate, num_logs)
 
     
     @staticmethod
     def check_login_callback(downloader):
-        url = 'http://www.geocaching.com/seek/nearest.aspx'
-        page = downloader.get_reader(url, login = False).read()
+        login_request = downloader.get_reader(GeocachingComCacheDownloader.NEAREST_URL, login = False)
+        page = login_request.read()
+        login_request.close()
 
         t = unicode(page, 'utf-8')
         doc = fromstring(t)
@@ -266,7 +284,6 @@ class GeocachingComCacheDownloader(CacheDownloader):
         
     @staticmethod
     def login_callback(downloader, username, password):
-        url = 'https://www.geocaching.com/login/default.aspx'
         values = {'ctl00$ContentBody$tbUsername': username,
             'ctl00$ContentBody$tbPassword': password,
             'ctl00$ContentBody$cbRememberMe': 'on',
@@ -274,7 +291,9 @@ class GeocachingComCacheDownloader(CacheDownloader):
             '__EVENTTARGET': '',
             '__EVENTARGUMENT': ''
         }
-        page = downloader.get_reader(url, values, login = False).read()
+        request = downloader.get_reader(GeocachingComCacheDownloader.LOGIN_URL, values, login = False)
+        page = request.read()
+        request.close()
 
         t = unicode(page, 'utf-8')
         doc = fromstring(t)
@@ -287,7 +306,7 @@ class GeocachingComCacheDownloader(CacheDownloader):
         raise Exception("Name/Password MAY be correct, but I encountered unexpected data while logging in.")
         
     def _get_user_token(self):
-        page = self.downloader.get_reader('http://www.geocaching.com/map/default.aspx?lat=6&lng=9', login_callback = self.login_callback, check_login_callback = self.check_login_callback)
+        page = self.downloader.get_reader(self.USER_TOKEN_URL, login_callback = self.login_callback, check_login_callback = self.check_login_callback)
         for line in page:
             if line.startswith('vxar uvtoken'):
                 user_token[0] = re.sub('[^A-Z0-9]+', '', line.split('=')[-1])
@@ -295,8 +314,9 @@ class GeocachingComCacheDownloader(CacheDownloader):
                 return
         logger.error("Using fallback for user token search!")
         try:
-            page = self.downloader.get_reader('http://www.geocaching.com/map/default.aspx?lat=6&lng=9', login_callback = self.login_callback, check_login_callback = self.check_login_callback)
+            page = self.downloader.get_reader(self.USER_TOKEN_URL, login_callback = self.login_callback, check_login_callback = self.check_login_callback)
             t = page.read()
+            page.close()
             user_token[0] = re.search('[A-Z0-9]{128}', t).group(0)
         except Exception:
             raise Exception("Website contents unexpected. Please check connection.")
@@ -304,9 +324,10 @@ class GeocachingComCacheDownloader(CacheDownloader):
     def _parse_cache_page(self, cache_page, coordinate, num_logs, download_images = True):
         logger.debug("Start parsing.")
         pg = cache_page.read()
+        cache_page.close()
         t = unicode(pg, 'utf-8')
         doc = fromstring(t)
-        
+                
         # Basename - Image name without path and extension
         def basename(url):
             return url.split('/')[-1].split('.')[0]
@@ -315,8 +336,8 @@ class GeocachingComCacheDownloader(CacheDownloader):
         try:
             coordinate.title = doc.cssselect('meta[name="og:title"]')[0].get('content')
         except Exception, e:
-            logger.error("Could not find title!")
-            raise e  
+            logger.error("Could not find title - cache is probably unpublished!")
+            raise Exception("Geocache not found.")
             
         # Type 
         try:
@@ -370,7 +391,7 @@ class GeocachingComCacheDownloader(CacheDownloader):
         
         # Owner
         try:
-            coordinate.owner = doc.cssselect('#cacheDetails span.minorCacheDetails a')[0].text_content()
+            coordinate.owner = doc.cssselect('#cacheDetails .minorCacheDetails a')[0].text_content()
         except Exception, e:
             logger.error("Owner not found!")
             raise e
@@ -394,26 +415,47 @@ class GeocachingComCacheDownloader(CacheDownloader):
         coordinate.set_waypoints(waypoints)
         
         # User token and Logs
+        userToken = ''
         for x in doc.cssselect('script'):
             if not x.text:
                 continue
             s = x.text.strip()
-            if s.startswith('//<![CDATA[\r\ninitalLogs'):
-                logs = s.replace('//<![CDATA[\r\ninitalLogs = ', '')
-                logs = re.sub('(?s)};\s*//]]>', '}', logs)
-                coordinate.set_logs(self._parse_logs_json(logs))
-            # User Token is not currently in use
-            #elif s.startswith('//<![CDATA[\r\nvar uvtoken'):
-            #    userToken = re.sub("(?s).*userToken = '", '', s)
-            #    userToken = re.sub("(?s)'.*", '', userToken)
-            #    print userToken
-            
+            if s.startswith('//<![CDATA[\r\nvar uvtoken'):
+                userToken = re.sub("(?s).*userToken = '", '', s)
+                userToken = re.sub("(?s)'.*", '', userToken)
+                print "userToken:", userToken
+
+        #Ask first page of logs. And same time number of pages
+        logs = self.downloader.get_reader(
+            self.LOGBOOK_URL % (userToken, 1),
+            login_callback = self.login_callback, 
+            check_login_callback = self.check_login_callback)
+        new_set_of_logs, total_page = self._parse_logs_json(logs.read(), True) #True=we want also number of page
+        logs.close()
+
+        page_of_logs=num_logs/10 #num_logs from parameter (which comes from settings 'download_num_logs')
+
+        #First page is already handled, so counter starts from 2
+        counter = 2
+        while (counter <= total_page and counter <= page_of_logs):
+            logs = self.downloader.get_reader(
+                self.LOGBOOK_URL % (userToken, counter),
+                login_callback = self.login_callback, 
+                check_login_callback = self.check_login_callback)
+            new_set_of_logs.extend(self._parse_logs_json(logs.read()))
+            logs.close()
+
+            counter += 1
+
+        coordinate.set_logs(new_set_of_logs)
+
         # Attributes
-        try:
-            coordinate.attributes = ','.join([x.get('title') for x in doc.cssselect('.CacheDetailNavigationWidget.BottomSpacing .WidgetBody img') if x.get('title') != 'blank'])
-        except Exception, e:
-            logger.error("Could not find/parse attributes")
-            raise e
+        ###These are loaded when preview is loaded.
+        #try:
+        #    coordinate.attributes = ','.join([x.get('title') for x in doc.cssselect('.CacheDetailNavigationWidget.BottomSpacing .WidgetBody img') if x.get('title') != 'blank'])
+        #except Exception, e:
+        #    logger.error("Could not find/parse attributes")
+        #    raise e
         
         # Image Handling
 
@@ -512,6 +554,7 @@ class GeocachingComCacheDownloader(CacheDownloader):
     def _parse_cache_page_print(self, cache_page, coordinate, num_logs):
         logger.debug("Start parsing.")
         pg = cache_page.read()
+        cache_page.close()
         t = unicode(pg, 'utf-8')
         doc = fromstring(t)
         
@@ -587,10 +630,38 @@ class GeocachingComCacheDownloader(CacheDownloader):
             
         # Attributes
         try:
-            coordinate.attributes = ','.join([x.get('title') for x in doc.cssselect('#Content .sortables .item-content')[5].cssselect('img') if x.get('title') != 'blank'])
+            for x in doc.cssselect('#Content .sortables .item-content')[5].cssselect('img'):
+                if x.get('title') == 'blank':
+                    continue
+
+                attrib=x.get('src')[19:] #strip text '/images/attributes/'
+
+                # Prepend local path to filename, and check do we have it already
+                filename = os.path.join(self.path, attrib)
+                if os.path.isfile(filename):
+                    logger.info("%s exists already, don't reload " % filename)
+                else:
+                    # Download file
+                    url="http://www.geocaching.com/images/attributes/%s" % attrib
+                    logger.info("Downloading %s to %s" % (url, filename))
+
+                    try:
+                        f = open(filename, 'wb')
+                        f.write(self.downloader.get_reader(url, login = False).read())
+                        f.close()
+                    except Exception, e:
+                        logger.exception(e)
+                        logger.error("Failed to download image from URL %s" % url)
+
+                #store filename without path to the comma separated string
+                coordinate.attributes = coordinate.attributes+attrib+","
+        except IndexError:
+            # There are no attributes
+            logger.info("Attributes not found!")
         except Exception, e:
             logger.error("Could not find/parse attributes")
             raise e
+            
 
         # Extract description...
         try:
@@ -644,10 +715,11 @@ class GeocachingComCacheDownloader(CacheDownloader):
             notes.append('%s,%sT10:00Z,%s,"%s"' % (geocache.name, geocache.logdate, log, text))
             
         logger.info("Uploading fieldnotes...")
-        url = 'http://www.geocaching.com/my/uploadfieldnotes.aspx'
         
         # First, download webpage to get the correct viewstate value
-        pg = self.downloader.get_reader(url, login_callback = self.login_callback, check_login_callback = self.check_login_callback).read()
+        cache_page = self.downloader.get_reader(self.UPLOAD_FIELDNOTES_URL, login_callback = self.login_callback, check_login_callback = self.check_login_callback)
+        pg = cache_page.read()
+        cache_page.close()
         t = unicode(pg, 'utf-8')
         doc = fromstring(t)
         # Sometimes this field is not available
@@ -665,6 +737,7 @@ class GeocachingComCacheDownloader(CacheDownloader):
             check_login_callback = self.check_login_callback)
 
         res = response.read()
+        response.close()
         t = unicode(res, 'utf-8')
         doc = fromstring(t)
         
@@ -697,7 +770,9 @@ class GeocachingComCacheDownloader(CacheDownloader):
             url = 'http://www.geocaching.com/seek/log.aspx?wp=%s' % geocache.name
             
             # First, download webpage to get the correct viewstate value
-            pg = self.downloader.get_reader(url, login_callback = self.login_callback, check_login_callback = self.check_login_callback).read()
+            cache_page = self.downloader.get_reader(url, login_callback = self.login_callback, check_login_callback = self.check_login_callback)
+            pg = cache_page.read()
+            cache_page.close()
             t = unicode(pg, 'utf-8')
             doc = fromstring(t)
             doc.forms[0].fields['ctl00$ContentBody$LogBookPanel1$ddLogType'] = str(log)
@@ -714,6 +789,7 @@ class GeocachingComCacheDownloader(CacheDownloader):
                 check_login_callback = self.check_login_callback)
 
             res = response.read()
+            res.close()
             t = unicode(res, 'utf-8')
             print t
             doc = fromstring(t)
@@ -759,11 +835,12 @@ class GeocachingComCacheDownloader(CacheDownloader):
             hints = re.sub(r'\[([^\]]+)\]', lambda match: HTMLManipulations._rot13(match.group(0)), hints)
         return hints
         
-    def _parse_logs_json(self, logs):
+    def _parse_logs_json(self, logs, returnTotalPages=False):
         try:
             r = json.loads(logs)
-        except Exception:
+        except Exception, e:
             logger.exception('Could not json-parse logs!')
+            raise e
         if not 'status' in r or r['status'] != 'success':
             logger.error('Could not read logs, status is "%s"' % r['status'])
         data = r['data']
@@ -776,9 +853,13 @@ class GeocachingComCacheDownloader(CacheDownloader):
             text = HTMLManipulations._decode_htmlentities(HTMLManipulations._strip_html(HTMLManipulations._replace_br(l['LogText'])))
             output.append(dict(type=tpe, date=date, finder=finder, text=text))
         logger.debug("Read %d log entries" % len(output))
-        return output
-        
 
+        if returnTotalPages:
+            pageinfo = r['pageInfo']
+            total_page = pageinfo['totalPages']
+            return output,total_page
+        
+        return output
 
 BACKENDS = {
     'geocaching-com-new': {'class': GeocachingComCacheDownloader, 'name': 'geocaching.com', 'description': 'Backend for geocaching.com'},
